/*
 * xvpic.c -- enfle xvpic handler
 * (C)Copyright 1999 by Hiroshi Takekawa
 * This file is part of Enfle.
 *
 * Last Modified: Wed Oct 13 06:31:53 1999.
 * $Id$
 *
 * Enfle is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Enfle is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#include <stdio.h>
#include <ctype.h>

#include "enfle.h"
#include "archive.h"
#include "xvpic.h"
#include "plugin.h"

int
#ifdef PIC
get_plugininfo(PluginInfo *p)
#else
loader_xvpic_get_plugininfo(PluginInfo *p)
#endif
{
  p->version = 1;
  p->type = _Loader;
  p->pluginname = "XVPIC Format Loader Plugin version 0.1";
  p->pluginshortname = FORMAT_NAME;
  p->author = "Hiroshi Takekawa";
  p->dlhandle = NULL; /* set by plugin_load */
  p->functions.loader.load_image = xvpic_load_image;

  return 1;
}

static char *
get_token(Archive *ar)
{
  static char line[256];
  static int last = 0;
  static int index = 0;
  int start, len;
  char *ret;

  if (index >= last) {
    if (archive_gets(ar, line, 256) == NULL)
      fprintf(stderr, "archive_gets() returns NULL\n");
    last = strlen(line);
    index = 0;
  }

  while (index < last && isspace(line[index]))
    index++;

  start = index;
  while (index < last && !isspace(line[index]))
    index++;
  len = index - start;

  if ((ret = malloc(len + 1)) == NULL)
    return NULL;
  strncpy(ret, line + start, len);
  ret[len] = '\0';

  while (index < last && isspace(line[index]))
    index++;

  return ret;
}

static int
xvpic_read_header(Info *info, Image *p)
{
  Archive *ar = info->ar;
  char *token;
  char buf[256];

  /* check ID */
  if (archive_gets(ar, buf, 256) == NULL)
    return 0;
  if (strncmp(buf, "P7 332", 6))
    return 0;

  /* read comment */
  for (;;) {
    if (archive_gets(ar, buf, 256) == NULL)
      return 0;
    if (!strncmp(buf, "#END_OF_COMMENTS", 16))
      break;
    if (!strncmp(buf, "#BUILDIN:", 9))
      return 0;
  }

  p->type = _INDEX;

  if ((token = get_token(ar)) == NULL)
    return 0;
  p->width = atoi(token);
  free(token);

  if ((token = get_token(ar)) == NULL)
    return 0;
  p->height = atoi(token);
  free(token);

  if ((token = get_token(ar)) == NULL)
    return 0;
  p->ncolors = atoi(token) + 1;
  free(token);

  if (p->width <= 0 || p->height <= 0 || p->ncolors <= 0)
    return 0;

  info->format = FORMAT_NAME;

#ifdef DEBUG
  fprintf(stderr, "xvpic_read_header: %d %d %dx%d\n", p->type, p->ncolors, p->width, p->height);
#endif

  return 1;
}

static int
xvpic_decode_image(Info *info, Image *p)
{
  Archive *ar = info->ar;
  int i;
  unsigned char *d;

#ifdef DEBUG
  fprintf(stderr, "xvpic_decode_image() called\n");
#endif

  if (!xvpic_read_header(info, p))
    return 0;

  /* create colormap */
  for (i = 0; i < p->ncolors; i++) {
    p->colormap[i][0] = i & 0xe0;
    p->colormap[i][1] = (i & 0x1c) << 3;
    p->colormap[i][2] = (i & 3) << 6;
  }

  p->bytes_per_line = image_calculate_bytes_per_line(p->width, p->type);
  p->image_size = p->bytes_per_line * p->height;
#ifdef DEBUG
  fprintf(stderr, "allocate memory %d bytes for image\n", p->image_size);
#endif
  if ((p->image = d = malloc(p->image_size)) == NULL)
    return 0;

  archive_read(ar, p->image, p->image_size);

  return 1;
}

int
xvpic_load_image(Info *info, Image *p)
{
  if (!xvpic_decode_image(info, p))
    return info->format == NULL ? PROC_NOT : PROC_ERROR;

  info->npics = 1;

  /* set handler */
  info->handler.destroy  = NULL;
  info->handler.alarmset = NULL;
  info->handler.alarm    = NULL;
  info->handler.alarmoff = NULL;

  return PROC_OK;
}
